import { TuyaDevice } from './baseContainers';
import { NO_VALUE } from '../container_utils';

export class DoorSensor extends TuyaDevice {
  private _doorContactState: boolean;
  private _batteryPercentage: number;

  public constructor(deviceID: string) {
    super(deviceID);
    this._doorContactState = false;
    this._batteryPercentage = NO_VALUE;
  }

  // ======================= GETTERS & SETTERS =======================

  public get doorContactState(): boolean {
    return this._doorContactState;
  }
  public set doorContactState(value: boolean) {
    this._doorContactState = value;
  }

  public get batteryPercentage(): number {
    return this._batteryPercentage;
  }
  public set batteryPercentage(value: number) {
    this._batteryPercentage = value;
  }

  // ======================= API =======================

  public async retrieveGetRequestDeviceData(tuyaCloudResponse: string) {
    await this.retrieveGetRequestGeneralData(tuyaCloudResponse);

    const doorContactStateMatch = this.cloudResponseString.match(/"code":"doorcontact_state","value":(\w+)/);
    const doorContactState = doorContactStateMatch ? doorContactStateMatch[1] == 'true' : null;
    if (doorContactState == null) throw new Error('Could not retrieve the door contact state!');

    this.doorContactState = doorContactState;

    const batteryPercentageMatch = this.cloudResponseString.match(/"code":"battery_percentage","value":(\d+)/);
    const batteryPercentage = batteryPercentageMatch ? batteryPercentageMatch[1] : NO_VALUE;
    if (+batteryPercentage == NO_VALUE) throw new Error('Could not retrieve the battery percentage!');

    this.batteryPercentage = +batteryPercentage;
  }

  public stringifyData(): string {
    const jsonData = {
      requestSuccess: this.tuyaGetRequestGeneralData.success,
      requestTime: this.tuyaGetRequestGeneralData.timestamp,
      deviceName: this.tuyaGetRequestGeneralData.deviceName,
      deviceStatus: this.tuyaGetRequestGeneralData.deviceStatus,
      updateTime: this.tuyaGetRequestGeneralData.updateTime,
      doorContactState: this.doorContactState,
      batteryPercentage: this.batteryPercentage,
    };

    return JSON.stringify(jsonData);
  }

  public printDataToConsole() {
    console.log('=================================================');
    console.log('Request success: ' + this.tuyaGetRequestGeneralData.success);
    console.log('Device name: ' + this.tuyaGetRequestGeneralData.deviceName);
    console.log('Device status: ' + this.tuyaGetRequestGeneralData.deviceStatus);
    console.log('Time: ' + this.tuyaGetRequestGeneralData.timestamp);
    console.log('Update time: ' + this.tuyaGetRequestGeneralData.updateTime);
    console.log('Door Contact State: ' + this.doorContactState);
    console.log('Battery: ' + this.batteryPercentage);
    console.log('=================================================');
  }
}
