import { TuyaDevice } from './baseContainers';
import { NO_VALUE } from '../container_utils';

export class SmartPlug extends TuyaDevice {
  private _current: number; // mA
  private _power: string; // W
  private _voltage: string; // V

  public constructor(deviceID: string) {
    super(deviceID);
    this._current = NO_VALUE;
    this._power = NO_VALUE.toString();
    this._voltage = NO_VALUE.toString();
  }

  // ======================= GETTERS & SETTERS =======================

  public get current(): number {
    return this._current;
  }
  public set current(value: number) {
    this._current = value;
  }

  public get power(): string {
    return this._power;
  }
  public set power(value: string) {
    this._power = value;
  }

  public get voltage(): string {
    return this._voltage;
  }
  public set voltage(value: string) {
    this._voltage = value;
  }

  // ======================= API =======================

  public async retrieveGetRequestDeviceData(tuyaCloudResponse: string) {
    await this.retrieveGetRequestGeneralData(tuyaCloudResponse);

    const currentMatch = this.cloudResponseString.match(/"code":"cur_current","value":(\d+)/);
    const current = currentMatch ? +currentMatch[1] : NO_VALUE;
    if (+current == NO_VALUE) throw new Error('Could not retrieve the current value!');
    this.current = current;

    const voltageMatch = this.cloudResponseString.match(/"code":"cur_voltage","value":(\d+)/);
    let voltage = voltageMatch ? voltageMatch[1] : NO_VALUE.toString();
    if (voltage == NO_VALUE.toString()) throw new Error('Could not retrieve the voltage value!');
    voltage = voltage.substring(0, voltage.length - 1) + '.' + voltage.substring(voltage.length - 1);
    this.voltage = voltage;

    const powerMatch = this.cloudResponseString.match(/"code":"cur_power","value":(\d+)/);
    let power = powerMatch ? powerMatch[1] : NO_VALUE.toString();
    if (power == NO_VALUE.toString()) throw new Error('Could not retrieve the consumed power value!');
    power = power.substring(0, power.length - 1) + '.' + power.substring(power.length - 1);
    this.power = power;
  }

  public stringifyData(): string {
    const jsonData = {
      requestSuccess: this.tuyaGetRequestGeneralData.success,
      requestTime: this.tuyaGetRequestGeneralData.timestamp,
      deviceName: this.tuyaGetRequestGeneralData.deviceName,
      deviceStatus: this.tuyaGetRequestGeneralData.deviceStatus,
      updateTime: this.tuyaGetRequestGeneralData.updateTime,
      current: this.current,
      power: this.power,
      voltage: this.voltage,
    };

    return JSON.stringify(jsonData);
  }

  public printDataToConsole() {
    console.log('=================================================');
    console.log('Request success: ' + this.tuyaGetRequestGeneralData.success);
    console.log('Device name: ' + this.tuyaGetRequestGeneralData.deviceName);
    console.log('Device status: ' + this.tuyaGetRequestGeneralData.deviceStatus);
    console.log('Time: ' + this.tuyaGetRequestGeneralData.timestamp);
    console.log('Update time: ' + this.tuyaGetRequestGeneralData.updateTime);
    console.log('Current value: ' + this.current);
    console.log('Voltage value: ' + this.voltage);
    console.log('Power consumed: ' + this.power);
    console.log('=================================================');
  }
}
