import { TuyaDevice } from './baseContainers';
import { NO_VALUE } from '../container_utils';

export class TemperatureSensor extends TuyaDevice {
  private _temperature: string;
  private _humidityPercentage: number;
  private _batteryPercentage: number;

  public constructor(deviceID: string) {
    super(deviceID);
    this._temperature = NO_VALUE.toString();
    this._humidityPercentage = NO_VALUE;
    this._batteryPercentage = NO_VALUE;
  }

  // ======================= GETTERS & SETTERS =======================

  public get temperature(): string {
    return this._temperature;
  }

  public set temperature(value: string) {
    this._temperature = value;
  }

  public get humidityPercentage(): number {
    return this._humidityPercentage;
  }

  public set humidityPercentage(value: number) {
    this._humidityPercentage = value;
  }

  public get batteryPercentage(): number {
    return this._batteryPercentage;
  }

  public set batteryPercentage(value: number) {
    this._batteryPercentage = value;
  }

  public setSensorData(temperature: string, humidityPercentage: number, batteryPercentage: number) {
    this.temperature = temperature;
    this.humidityPercentage = humidityPercentage;
    this.batteryPercentage = batteryPercentage;
  }

  // ======================= API =======================

  public async retrieveGetRequestDeviceData(tuyaCloudResponse: string) {
    await this.retrieveGetRequestGeneralData(tuyaCloudResponse);

    const temperatureMatch = this.cloudResponseString.match(/"code":"va_temperature","value":(\d+)/);
    let temperature = temperatureMatch ? temperatureMatch[1] : NO_VALUE.toString();
    if (+temperature == NO_VALUE) throw new Error('Could not retrieve the temperature!');
    temperature =
      temperature.substring(0, temperature.length - 1) + '.' + temperature.substring(temperature.length - 1);

    const humidityPercentageMatch = this.cloudResponseString.match(/"code":"va_humidity","value":(\d+)/);
    const humidityPercentage = humidityPercentageMatch ? humidityPercentageMatch[1] : NO_VALUE;
    if (+humidityPercentage == NO_VALUE) throw new Error('Could not retrieve the humidity percentage!');

    const batteryPercentageMatch = this.cloudResponseString.match(/"code":"battery_percentage","value":(\d+)/);
    const batteryPercentage = batteryPercentageMatch ? batteryPercentageMatch[1] : NO_VALUE;
    if (+batteryPercentage == NO_VALUE) throw new Error('Could not retrieve the battery percentage!');

    this.setSensorData(temperature, +humidityPercentage, +batteryPercentage);
  }

  public stringifyData(): string {
    const jsonData = {
      requestSuccess: this.tuyaGetRequestGeneralData.success,
      requestTime: this.tuyaGetRequestGeneralData.timestamp,
      deviceName: this.tuyaGetRequestGeneralData.deviceName,
      deviceStatus: this.tuyaGetRequestGeneralData.deviceStatus,
      updateTime: this.tuyaGetRequestGeneralData.updateTime,
      temperature: this.temperature,
      humidityPercentage: this.humidityPercentage,
      batteryPercentage: this.batteryPercentage,
    };

    return JSON.stringify(jsonData);
  }

  public printDataToConsole() {
    console.log('=================================================');
    console.log('Request success: ' + this.tuyaGetRequestGeneralData.success);
    console.log('Device name: ' + this.tuyaGetRequestGeneralData.deviceName);
    console.log('Device status: ' + this.tuyaGetRequestGeneralData.deviceStatus);
    console.log('Time: ' + this.tuyaGetRequestGeneralData.timestamp);
    console.log('Update time: ' + this.tuyaGetRequestGeneralData.updateTime);
    console.log('Temperature: ' + this.temperature);
    console.log('Humidity: ' + this.humidityPercentage);
    console.log('Battery: ' + this.batteryPercentage);
    console.log('=================================================');
  }
}
